#ifndef __SORT_H
#define __SORT_H

#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>

/*
 * Employee info struct
 */
typedef struct {
    int id;
    int salary;
} employee_t;

/*
 * Generic data type.
 */
typedef employee_t* data_t;

/*
 * Compares two data_t values, returning -1 if a < b, 1 if a > b, and 0 if
 * they are equal.
 */
int cmp(data_t a, data_t b);

/*
 * Generic sorting routine type (function pointer).
 */
typedef void(*sort)(data_t[], size_t);

/*
 * Sort an array using bubble sort.
 */
void bubble_sort(data_t items[], size_t n);

/*
 * Sort an array using selection sort.
 */
void selection_sort(data_t items[], size_t n);

/*
 * Sort an array using insertion sort.
 */
void insertion_sort(data_t items[], size_t n);

/*
 * Sort an array using merge-sort.
 */
void merge_sort(data_t items[], size_t n);

/*
 * Sort an array using quicksort.
 */
void quick_sort(data_t items[], size_t n);

/*
 * Sort an array using modified insertion sort.
 */
void insertion_sort_binary(data_t items[], size_t n);

/*
 * Sort an array using modified merge-sort.
 */
void merge_sort_swap(data_t items[], size_t n);

/*
 * Sort an array using modified quicksort.
 */
void quick_sort_midpoint(data_t items[], size_t n);

#endif

